SystemOrganization addCategory: #'GT-Debugger'!

!PreDebugWindow methodsFor: '*gt-debugger' stamp: 'TudorGirba 11/1/2012 21:23'!
spawnGTDebugger
	model spawnGTDebugger! !

Debugger subclass: #GTProxyDebugger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger'!

!GTProxyDebugger commentStamp: 'TudorGirba 11/2/2012 22:04' prior: 0!
GTProxyDebugger is a simple extension of the existing debugger that adds a button for "debug glamorously" using the GTDebugger in the regular debugger.

The goal is to provide the possibility of having the GTDebugger next to the classic debugger until the inevitable errors are ironed out.!

!GTProxyDebugger classSide methodsFor: 'class initialization' stamp: 'AndreiChis 11/1/2012 15:13'!
initialize
	super initialize.

	"Add ourselves to registry. See [Smalltalk tools]" 
	Smalltalk tools register: self as: #debugger! !

!GTProxyDebugger methodsFor: 'accessing' stamp: 'AndreiChis 11/25/2012 12:52'!
debuggerClass
	^ GTDebugger ! !

!GTProxyDebugger methodsFor: 'initialize' stamp: 'TudorGirba 11/1/2012 21:23'!
preDebugButtonQuads
	| buttons |
	self isPostMortem 
		ifTrue: [ ^ self postMortemPreDebugButtonQuads ].
	^ {
		{'Proceed' translated.	#proceed. 	#blue. 	'Continue execution' translated}.
		{'Abandon' translated.	#abandon. 	#black.	'Abandon this execution by closing this window' translated}.
		{'Debug'    		translated.		#debug.		#red. 	'Bring up a debugger' translated}.
		{'Debug glamorously' translated. #spawnGTDebugger. #red. 'Bring up the Glamorous Debugger' translated}
	}! !

!GTProxyDebugger methodsFor: 'notifier menu' stamp: 'AndreiChis 1/15/2013 12:14'!
spawnGTDebugger
	| topView |
	topView := self topView.
	topView model: nil.
	"so close won't release me."
	self breakDependents.
	topView delete.
	
	self debuggerClass debugSession: ((self debuggerClass 
		spanNewSessionForContext: self interruptedContext fromProcess: self interruptedProcess)
			errorWasInUIProcess: errorWasInUIProcess) 
! !

GLMCompositePresentation subclass: #GTDebugger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger'!

!GTDebugger commentStamp: 'TudorGirba 11/2/2012 22:07' prior: 0!
GTDebugger is a browser that provides Smalltalk debugging functionality.

An example:
[ GTDebugger debugProcess: Processor activeProcess.
	Processor activeProcess suspend.
	#(1 2 3 4 5) do: [:each | each asString]
] fork.!

!GTDebugger classSide methodsFor: 'instance creation' stamp: 'AndreiChis 1/15/2013 12:07'!
debugContext: aContext fromProcess: aProcess

	self debugSession: (self spanNewSessionForContext: aContext fromProcess: aProcess)! !

!GTDebugger classSide methodsFor: 'instance creation' stamp: 'AndreiChis 11/25/2012 12:32'!
debugProcess: aProcess

	self debugContext: aProcess suspendedContext fromProcess: aProcess! !

!GTDebugger classSide methodsFor: 'instance creation' stamp: 'AndreiChis 1/28/2013 16:38'!
debugSession: aSession

	<primitive: 19> "simulation guard"

	"WorldState addDeferredUIMessage: ["
		self openOn: aSession 
	"]"! !

!GTDebugger classSide methodsFor: 'instance creation' stamp: 'TudorGirba 11/25/2012 23:00'!
openOn: anObject 
	^ self new openOn: anObject! !

!GTDebugger classSide methodsFor: 'private' stamp: 'AndreiChis 1/13/2013 20:19'!
sessionClass

	^ DebugSession ! !

!GTDebugger classSide methodsFor: 'accessing' stamp: 'AndreiChis 1/15/2013 12:07'!
spanNewSessionForContext: aContext fromProcess: aProcess

	^ self sessionClass process: aProcess context: aContext ! !

!GTDebugger classSide methodsFor: 'accessing' stamp: 'AndreiChis 1/28/2013 14:33'!
spanNewSessionForProcess: aProcess

	^ self spanNewSessionForContext: aProcess suspendedContext fromProcess: aProcess! !

!GTDebugger methodsFor: 'building actions' stamp: 'AndreiChis 1/15/2013 15:29'!
actOnBrowserClosing: ann
	self model terminate! !

!GTDebugger methodsFor: 'building actions' stamp: 'AndreiChis 1/14/2013 15:30'!
addCreateMessageActionFor: aPresentation to: aCollection

	self model isInterruptedContextDoesNotUnderstand ifTrue: [ 
		aCollection add:
			((GLMGenericAction new) 
				action: [ :presentation :model |
					model createMethodInContext: self selectedContext.
					self selectTopOfStack: model context ]; 
				icon: GLMUIThemeExtraIcons glamorousAdd;
				title: 'Create method';
				yourself) ]! !

!GTDebugger methodsFor: 'private' stamp: 'AndreiChis 11/28/2012 09:52'!
browser 
	^ self presentations first! !

!GTDebugger methodsFor: 'building' stamp: 'AndreiChis 1/15/2013 15:06'!
compose
	self title: 'Glamorous Debugger'.
	self tabulator with: [:browser | 
		browser
			row: [ :row | 
						row
							column: #stack;
							column: #code ];
			row: #inspector.
		browser transmit
			to: #stack;
			andShow: [ :composite | self stackIn: composite ].
		browser transmit
			fromOutsidePort: #entity;
			to: #stack port: #selection;
			transformed: [:model | model context].
		browser transmit
			from: #stack port: #selection;
			fromOutsidePort: #entity;
			to: #stack port: #context;
			transformed: [ :context :model | model createModelForContext: context ].
		browser transmit
			from: #stack port: #selection;
			to: #stack port: #pc;
			transformed: #pc.
		browser transmit
			from: #stack port: #context;
			fromOutsidePort: #entity;
			to: #code;
			andShow: [ :composite | self methodCodeIn: composite ].
		browser transmit
			from: #stack;
			fromOutsidePort: #entity;
			from: #stack port: #pc;
			to: #code port: #selectionInterval;
			transformed: [ :context :model :pc | model selectedCodeRange: context ].
		browser transmit
			from: #stack port: #context;
			to: #inspector;
			andShow: [ :composite | self inspectorForContextAndSelfIn: composite ] ].
	self on: GLMBrowserClosing send: #actOnBrowserClosing: to: self.! !

!GTDebugger methodsFor: 'building' stamp: 'TudorGirba 11/25/2012 23:02'!
inspectorForContextAndSelfIn: composite
	composite dynamic
		display: [ :each | 
			"We use the first actual finder because need to send showFirst: to it"
			GTInspector new first
				noTitle;
				noActions;
				showFirst: [ :a | a custom: (GTDebuggerVariablesBrowser new startOn: each) ];
				startOn: each ]! !

!GTDebugger methodsFor: 'building actions' stamp: 'AndreiChis 1/14/2013 17:43'!
installCodeActionsFor: aPresentation
	self installWhereIsActionFor: aPresentation.
	self installRunToHereActionFor: aPresentation.
	self installSaveActionFor: aPresentation.! !

!GTDebugger methodsFor: 'building actions' stamp: 'AndreiChis 1/14/2013 17:44'!
installDebuggingActionsFor: aPresentation
	self installRestartActionFor: aPresentation.
	self installStepIntoActionFor: aPresentation.
	self installStepOverActionFor: aPresentation.
	self installStepThroughActionFor: aPresentation.
	self installResumeActionFor: aPresentation.
	self installDynamicActionsFor: aPresentation.! !

!GTDebugger methodsFor: 'building actions' stamp: 'AndreiChis 1/14/2013 16:10'!
installDynamicActionsFor: aPresentation
	aPresentation dynamicActions: [ :presentation || newActions |
		newActions := OrderedCollection new.
		self addCreateMessageActionFor: aPresentation to: newActions.
		newActions ]! !

!GTDebugger methodsFor: 'building actions' stamp: 'TudorGirba 11/2/2012 06:14'!
installRestartActionFor: aPresentation 
	aPresentation 
		act: [ :presentation :model |
			model restart: self selectedContext.
			self selectTopOfStack: model context  ]
		icon: GLMUIThemeExtraIcons glamorousRestart
		entitled: 'Restart'! !

!GTDebugger methodsFor: 'building actions' stamp: 'AndreiChis 1/15/2013 14:53'!
installResumeActionFor: aPresentation
	aPresentation 
		act: [ :presentation :model |
			model resume.
			model clear.
			self browser close. ]
		icon: GLMUIThemeExtraIcons glamorousPlay
		entitled: 'Resume' ! !

!GTDebugger methodsFor: 'building actions' stamp: 'AndreiChis 1/14/2013 17:43'!
installRunToHereActionFor: aPresentation
	aPresentation 
		act: [ :text :contextModel | 
			self selectionInterval ifNotNil: [:selectionInterval | 
				self model runToSelection: selectionInterval inContext: self selectedContext.
				self selectTopOfStack: self model context 	] ]
		icon: GLMUIThemeExtraIcons glamorousPlay
		entitled: 'Run to Here' ! !

!GTDebugger methodsFor: 'building actions' stamp: 'AndreiChis 1/14/2013 17:44'!
installSaveActionFor: aPresentation 
	aPresentation 
		act: [ :text :contextModel | 			
			 (self model recompileMethodTo: text text inContext: self selectedContext)
				 ifNotNil: [ self selectTopOfStack: self model context ] ] 
		icon: GLMUIThemeExtraIcons glamorousAccept 
		on: $s
		entitled: 'Accept'! !

!GTDebugger methodsFor: 'building actions' stamp: 'TudorGirba 11/2/2012 06:15'!
installStepIntoActionFor: aPresentation 
	aPresentation 
		act: [ :presentation :model |
			model stepInto: self selectedContext.
			self selectTopOfStack: model context ]
		icon: GLMUIThemeExtraIcons glamorousInto
		entitled: 'Step into' ! !

!GTDebugger methodsFor: 'building actions' stamp: 'TudorGirba 11/2/2012 06:15'!
installStepOverActionFor: aPresentation
	aPresentation 
		act: [ :presentation :model |
			model stepOver: self selectedContext.
			self selectTopOfStack: model context ]
		icon: GLMUIThemeExtraIcons glamorousOver
		entitled: 'Step over' ! !

!GTDebugger methodsFor: 'building actions' stamp: 'TudorGirba 11/2/2012 06:15'!
installStepThroughActionFor: aPresentation
	aPresentation 
		act: [ :presentation :model |
			model stepThrough: self selectedContext.
			self selectTopOfStack: model context ]
		icon: GLMUIThemeExtraIcons glamorousThrough
		entitled: 'Step through' ! !

!GTDebugger methodsFor: 'building actions' stamp: 'AndreiChis 1/14/2013 17:44'!
installWhereIsActionFor: aPresentation 
	aPresentation 
		act: [ :text :contextModel | 
			self selectionInterval: (self model pcRangeForContext:  self selectedContext ) ]
		icon: GLMUIThemeExtraIcons glamorousSearch 
		on: $w
		entitled: 'Where is?'! !

!GTDebugger methodsFor: 'private' stamp: 'AndreiChis 10/23/2012 15:56'!
keyAsFormatedString: key

	^ [ Text fromString: (key asString contractTo: 50) ]
			on: Error
			do: [ Text string: 'error printing' attribute: TextColor red ] ! !

!GTDebugger methodsFor: 'building' stamp: 'AndreiChis 1/14/2013 17:48'!
methodCodeIn: composite
	composite smalltalkCode
		doItReceiver: [:contextModel | contextModel receiver ];
		title: 'Source';
		format: #sourceText;
		smalltalkClass: [ :contextModel | contextModel selectedClass ];
		with: [ :code | self installCodeActionsFor: code ]! !

!GTDebugger methodsFor: 'accessing' stamp: 'AndreiChis 11/28/2012 09:54'!
model
	^ (self browser pane port: #entity) value! !

!GTDebugger methodsFor: 'private' stamp: 'AndreiChis 1/14/2013 15:00'!
selectTopOfStack: aContext
	| stackPane |
	stackPane := self browser paneNamed: #stack.
	(stackPane port: #selection) value: aContext.
	(stackPane port: #pc) value: aContext pc.
	stackPane update.! !

!GTDebugger methodsFor: 'accessing' stamp: 'AndreiChis 11/28/2012 09:54'!
selectedContext
	^ ((self browser paneNamed: #stack) port: #selection) value! !

!GTDebugger methodsFor: 'private' stamp: 'AndreiChis 1/13/2013 19:27'!
selectionInterval
	^ ((self browser paneNamed: #code) port: #selectionInterval) value! !

!GTDebugger methodsFor: 'private' stamp: 'AndreiChis 1/13/2013 20:07'!
selectionInterval: anInterval
	^ ((self browser paneNamed: #code) port: #selectionInterval) value: anInterval! !

!GTDebugger methodsFor: 'building' stamp: 'AndreiChis 11/25/2012 12:09'!
stackIn: composite
	composite list	
		title: 'Stack';
		showOnly: 10;
		display: #stack;
		format: [ :each | String streamContents: [:stream | each printDebugOn: stream ] ];
		with: [ :list | self installDebuggingActionsFor: list ]! !

!GTDebugger methodsFor: 'private' stamp: 'AndreiChis 10/23/2012 15:58'!
valueAsFormatedString: value

	^ [ Text fromString: value printString]
			on: Error
			do: [ Text string: 'error printing' attribute: TextColor red ] ! !

!GTDebugger methodsFor: 'events' stamp: 'AndreiChis 1/15/2013 15:08'!
windowIsClosing 
	self announce: GLMBrowserClosing new! !

GLMCompositePresentation subclass: #GTDebuggerVariablesBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger'!

!GTDebuggerVariablesBrowser commentStamp: 'TudorGirba 11/2/2012 22:06' prior: 0!
GTDebuggerVariablesBrowser is a special browser that is used in the first pane of the inspector that is embedded in the debugger. Its main intention is to provide a unified list of variables that come from:
- the instance variables of the receiver
- the local variables from the context!

!GTDebuggerVariablesBrowser methodsFor: 'building' stamp: 'TudorGirba 11/25/2012 22:57'!
compose  
	self title: [:each | GTInspector new printObjectAsAnItem: each receiver ].
	self tabulator with: [:browser | 
		browser 
			morphicAct: [:anotherComposite | anotherComposite entity receiver browse]
				icon: GLMUIThemeExtraIcons glamorousBrowse 
				entitled: 'Browse'.
		browser row: #variables; row: #evaluator.
		browser transmit to: #variables; andShow: [:a | 
			self variablesIn: a ].
		browser transmit to: #evaluator; andShow: [:a | 
			self evaluatorIn: a ].
		browser transmit from: #variables; toOutsidePort: #selection.
		browser transmit from: #evaluator; toOutsidePort: #selection ]! !

!GTDebuggerVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 10/30/2012 21:03'!
evaluatorIn: composite
	(composite smalltalkCode)
		doItReceiver: #yourself;
		display: [ :model | [model receiver printString asSmalltalkComment, String cr, 'self'] on: Error do: ['"Error printing. Try self printString to debug"'] ];
		selectionPopulate: #selection
			on: $o
			entitled: 'Do it and open (o)'
			with: [ :text :model | Compiler evaluate: text selectedText for: model receiver logged: false ]! !

!GTDebuggerVariablesBrowser methodsFor: 'private building' stamp: 'TudorGirba 11/2/2012 22:10'!
variablesIn: composite
	composite table
		showOnly: 50;
		display: [ :contextModel | contextModel gtDebuggerRetrieveVariableValuePairs ];
		sorted: [:variablePair1 :variablePair2 | variablePair1 key < variablePair2 key];
		column: 'Variable' evaluated: [:assoc | 
			[ Text fromString: assoc key asString ]
						on: Error
						do: [ Text string: 'error printing' attribute: TextColor red ] ];
		column: 'Value' evaluated: [:assoc | 
			[ Text fromString: (assoc value printString contractTo: 50) ]
						on: Error
						do: [ Text string: 'error printing' attribute: TextColor red ] ];
"		icon: [ :each |
			each value class theNonMetaClass browserIcon
				ifNotNil: [ OBMorphicIcons iconNamed: each value class theNonMetaClass browserIcon ] ];"
		send: #value;
		morphicSelectionAct: [:each | each value browse ] 
			icon: GLMUIThemeExtraIcons glamorousBrowse 
			on: $b 
			entitled: 'Browse'! !
GTProxyDebugger initialize!
